<?php

use CleverReach\Infrastructure\Interfaces\Required\Configuration;

/**
 * Configuration service.
 *
 * @see \CleverReach\Infrastructure\Interfaces\Required\Configuration
 */
class CleverReachConfigService extends Configuration {
  const CLIENT_ID = 'nRiRL1I0WF';
  const CLIENT_SECRET = 'tdOshpUD6LvSq9h0RiXklkcDcYFUYCAr';

  const INTEGRATION_NAME = 'Drupal';
  const SUBSCRIPTION_FIELD = 'field_cleverreach_subscribed';

  const CLEVERREACH_BUILD_EMAIL_URL = '/admin/mailing_create_new.php';

  /**
   * Return whether article/product search is enabled or not.
   *
   * @return bool
   *   For this integration is enabled always.
   */
  public function isProductSearchEnabled() {
    return TRUE;
  }

  /**
   * Gets site name defined in configuration.
   *
   * @return string
   *   Site name.
   */
  public function getSiteName() {
    return $this->getConfigRepository()->get('site_name');
  }

  /**
   * Retrieves parameters needed for article search registrations.
   *
   * @return array
   *   Keyed array, name, url, password.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function getProductSearchParameters() {
    $articleSearchUrl = url('cleverreach/search', array('absolute' => TRUE));

    return array(
      'url' => $articleSearchUrl,
      'name' => $this->getIntegrationListName(),
      'password' => $this->getArticleSearchEndpointPassword(),
    );
  }

  /**
   * Retrieves name of the CleverReach list (group).
   *
   * @return string
   *   CleverReach group name.
   */
  public function getIntegrationListName() {
    return "{$this->getIntegrationName()} - {$this->getSiteName()}";
  }

  /**
   * Retrieves integration name.
   *
   * @return string
   *   Integration name.
   */
  public function getIntegrationName() {
    return self::INTEGRATION_NAME;
  }

  /**
   * Get user information.
   *
   * @return array
   *   Keyed array with user info.
   */
  public function getUserInfo() {
    if (empty($this->userInfo)) {
      $this->userInfo = $this->getConfigRepository()
        ->get('CLEVERREACH_USER_INFO');
    }

    return $this->userInfo === NULL ? array() : $this->userInfo;
  }

  /**
   * Save user information in integration database.
   *
   * @param array $userInfo
   *   Keyed array with user info.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function setUserInfo($userInfo) {
    $this->getConfigRepository()->set('CLEVERREACH_USER_INFO', $userInfo);
    $this->userInfo = $userInfo;
  }

  /**
   * Gets task runner status information.
   *
   * @return array
   *   Key array containing guid and timestamp.
   */
  public function getTaskRunnerStatus() {
    /** @var array $runnerStatus */
    $runnerStatus = $this->getConfigRepository()
      ->get('CLEVERREACH_TASK_RUNNER_STATUS');
    return $runnerStatus === NULL ? array() : $runnerStatus;
  }

  /**
   * Sets task runner status information as array.
   *
   * @param string $guid
   *   Unique string.
   * @param int $timestamp
   *   Unix timestamp.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function setTaskRunnerStatus($guid, $timestamp) {
    $this->getConfigRepository()->set(
      'CLEVERREACH_TASK_RUNNER_STATUS',
      array(
        'guid' => $guid,
        'timestamp' => $timestamp,
      )
    );
  }

  /**
   * Gets CleverReach client id for Drupal integration.
   *
   * @return string
   *   Unique client id generated by CleverReach.
   */
  public function getClientId() {
    return self::CLIENT_ID;
  }

  /**
   * Gets CleverReach client secret for Drupal integration.
   *
   * @return string
   *   Unique client secret generated by CleverReach.
   */
  public function getClientSecret() {
    return self::CLIENT_SECRET;
  }

  /**
   * Return true if recipient is active by default, otherwise false.
   *
   * @return bool
   *   Default subscription behavior.
   */
  public function getDefaultRecipientStatus() {
    return $this->getConfigRepository()
      ->get('CLEVERREACH_DEFAULT_RECIPIENT_STATUS') === TRUE;
  }

  /**
   * Sets default recipient status.
   *
   * @param bool $status
   *   Default subscription behavior.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function setDefaultRecipientStatus($status) {
    $this->getConfigRepository()
      ->set('CLEVERREACH_DEFAULT_RECIPIENT_STATUS', $status);
  }

  /**
   * Checks if initial sync is configured.
   *
   * @return bool
   *   If initial sync is configured, returns true, otherwise false.
   */
  public function isConfiguredInitialSync() {
    return $this->getConfigRepository()
      ->get('CLEVERREACH_CONFIGURED_INITIAL_SYNC') === TRUE;
  }

  /**
   * Sets flag that indicate whether initial sync is configured or not.
   *
   * @param bool $value
   *   Indicator whether initial sync is configured or not.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function setConfiguredInitialSync($value = TRUE) {
    $this->getConfigRepository()
      ->set('CLEVERREACH_CONFIGURED_INITIAL_SYNC', $value);
  }

  /**
   * Gets article search endpoint password.
   *
   * @return string
   *   Unique article search password stored in database
   *   when module is installed.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function getArticleSearchEndpointPassword() {
    return $this->getConfigRepository()
      ->get('CLEVERREACH_ARTICLE_SEARCH_PASSWORD');
  }

  /**
   * Sets article search endpoint password.
   *
   * @param string $password
   *   Random generated string.
   *
   * @throws Exception
   *   When module is not installed.
   */
  public function setArticleSearchEndpointPassword($password) {
    $this->getConfigRepository()
      ->set('CLEVERREACH_ARTICLE_SEARCH_PASSWORD', $password);
  }

  /**
   * Gets queue name used for this integration.
   *
   * @return string
   *   Queue name used for integration.
   */
  public function getQueueName() {
    return 'drupalDefault';
  }

  /**
   * Returns URL to the event handler.
   *
   * @return string
   *   Event handler endpoint URL.
   */
  public function getCrEventHandlerURL() {
    return url(
      clever_reach_get_route_by_name('eventhandler'),
      array('absolute' => TRUE)
    );
  }

  /**
   * Gets number of synced recipients.
   *
   * @return int
   *   Number of synced recipients.
   */
  public function getNumberOfSyncedRecipients() {
    $result = $this->getConfigRepository()
      ->get('CLEVERREACH_NUMBER_OF_SYNCED_RECIPIENTS');

    return empty($result) ? 0 : (int) $result;
  }

}
